==> luna_cow-0.5.86-py3-none-any.whl: cow/utils/mixins.py <==
"""
Interface for "initializable" components.
"""

import asyncio

from async_timeout import timeout

from .log import logger

# tracks initialization states of the component (should not be used in cleanup() stage)
_initialized = {}


class InitializationError(Exception):
    """Indicates that there was an error during initialization."""


class Initializable:
    """
    Interface for "initializable" components of the application.

    Component should implement appropriate methods (initialize, close, probe).
    """

    # Default timeout is set to 30 because whole platform is intented to start within 30 secs,
    # and so do individual components.
    # Note, it works fine only for components without dependencies.
    INITIALIZE_TIMEOUT = 30

    def deps(self) -> list["Initializable"]:
        """
        Yields object attributes that pretend to be "initializable".
        """
        for value in self.__dict__.values():
            if isinstance(value, Initializable):
                yield value

    async def initializeWrapper(self):
        """
        Wraps components initialize() implementation.

        The following steps are done:
        1) Ensure no component is initialized 2+ times.
        2) Wait for initialization of the dependencies.
        3) Wait for component availability (probe).

        After all of the preceding steps succeeds run actual initialize().
        """
        _id = id(self)
        if _id in _initialized:
            await _initialized[_id].wait()
            return
        _initialized[_id] = asyncio.Event()

        # wait for dependencies to initialize (wait for initialized event)
        # doesn't handle cases with circullar deps
        coros = [x.initializeWrapper() for x in self.deps()]
        await asyncio.gather(*coros)

        try:
            if self.INITIALIZE_TIMEOUT:
                async with timeout(self.INITIALIZE_TIMEOUT):
                    await self._waitFor()
                    await self.initialize()
            else:
                await self._waitFor()
                await self.initialize()

        except asyncio.TimeoutError as exc:
            logger.error(f"Probing has failed for: {self.__class__.__name__}")
            raise InitializationError from exc

        _initialized[_id].set()

    async def initialize(self):
        """Placeholder for initialize()."""

    async def probe(self) -> bool:
        """Verify configuration."""
        return True

    async def close(self):
        """
        Placeholder for close() method.

        Component is fully responsible for cleanup stage.
        """

    async def _waitFor(self):
        """
        Given coroutine function wait for it to succeed.
        """
        name = self.__class__.__name__
        logger.debug(f"waiting for {name}")
        while True:
            res = await self.probe()
            if res:
                logger.debug(f"{name} is ready: {res}")
                break

            await asyncio.sleep(0.1)

==> luna_cow-0.8.10-py3-none-any.whl: cow/utils/mixins.py <==
"""
Interface for "initializable" components.
"""

import asyncio

from async_timeout import timeout

from .log import logger

# tracks initialization states of the component (should not be used in cleanup() stage)
_initialized = {}


class InitializationError(Exception):
    """Indicates that there was an error during initialization."""


class Initializable:
    """
    Interface for "initializable" components of the application.

    Component should implement appropriate methods (initialize, close, probe).
    """

    # Default timeout is set to 30 because whole platform is intented to start within 30 secs,
    # and so do individual components.
    # Note, it works fine only for components without dependencies.
    INITIALIZE_TIMEOUT = 30

    def deps(self) -> list["Initializable"]:
        """
        Yields object attributes that pretend to be "initializable".
        """
        for value in self.__dict__.values():
            if isinstance(value, Initializable):
                yield value

    async def initializeWrapper(self):
        """
        Wraps components initialize() implementation.

        The following steps are done:
        1) Ensure no component is initialized 2+ times.
        2) Wait for initialization of the dependencies.
        3) Wait for component availability (probe).

        After all of the preceding steps succeeds run actual initialize().
        """
        _id = id(self)
        if _id in _initialized:
            await _initialized[_id].wait()
            return
        _initialized[_id] = asyncio.Event()

        # wait for dependencies to initialize (wait for initialized event)
        # doesn't handle cases with circullar deps
        coros = [x.initializeWrapper() for x in self.deps()]
        await asyncio.gather(*coros)

        try:
            if self.INITIALIZE_TIMEOUT:
                async with timeout(self.INITIALIZE_TIMEOUT):
                    await self.initialize()
            else:
                await self.initialize()

        except asyncio.TimeoutError as exc:
            logger.error(f"Probing has failed for {self.__class__.__name__} within {self.INITIALIZE_TIMEOUT} seconds")
            raise InitializationError from exc

        _initialized[_id].set()

    async def initialize(self):
        """Placeholder for initialize()."""

    async def probe(self) -> bool:
        """Verify configuration."""
        return True

    async def close(self):
        """
        Placeholder for close() method.

        Component is fully responsible for cleanup stage.
        """
